@@ -46,6 +46,19 @@ class MetaFeatureLib:
             vis = utils.vis_point_uvs(image_points, vis)
         return vis
 
+    def vis_img(self, img_or_path, cam=None):
+        """
+        Direct vis img's feature
+        """
+        if isinstance(img_or_path, dict):
+            d = img_or_path
+        elif isinstance(img_or_path, np.ndarray):
+            d = dict(img=img_or_path)
+        elif isinstance(img_or_path, str):
+            d = dict(img=boxx.imread(img_or_path))
+        self.find_image_points(d)
+        return self.vis(d, cam)
+
     @classmethod
     def build_with_calibration_img(cls, **kwargs):
         # build init_kwargs and calibration_img code
@@ -53,7 +66,6 @@ class MetaFeatureLib:
             "Could go to https://calib.io/pages/camera-calibration-pattern-generator to genearte calibration image"
         )
         self = cls(**init_kwargs)
-        self.init_kwargs = init_kwargs
         self.calibration_img = calibration_img
         self.calibration_img_info = info
         self.calibration_img_name = "xx mm, h*w.png"
@@ -62,6 +74,7 @@ class MetaFeatureLib:
 
 class CheckboardFeatureLib(MetaFeatureLib):
     def __init__(self, checkboard=(11, 8), size_mm=25):
+        self.init_kwargs = dict(checkboard=checkboard, size_mm=size_mm)
         self.checkboard = checkboard
         self.size_mm = size_mm
         self.object_points = np.zeros(
@@ -91,15 +104,16 @@ class CheckboardFeatureLib(MetaFeatureLib):
         if cam is not None and "T" in d:
             vis = utils.vis_T(d["T"], cam, vis)
         h, w = vis.shape[:2]
-        draw_subpix = h < 1500
-        if draw_subpix:
-            sub_size = 2
-            vis = cv2.resize(vis, (w * sub_size, h * sub_size))
-            cv2.drawChessboardCorners(
-                vis, self.checkboard, d["corners"] * sub_size, True
-            )
-        else:
-            cv2.drawChessboardCorners(vis, self.checkboard, d["corners"], True)
+        if "corners" in d:
+            draw_subpix = max(h, w) < 800
+            if draw_subpix:
+                sub_size = 2
+                vis = cv2.resize(vis, (w * sub_size, h * sub_size))
+                cv2.drawChessboardCorners(
+                    vis, self.checkboard, d["corners"] * sub_size, True
+                )
+            else:
+                cv2.drawChessboardCorners(vis, self.checkboard, d["corners"], True)
         return vis
 
     @classmethod
@@ -222,7 +236,15 @@ class CharucoFeatureLib(MetaFeatureLib):
         square_size_mm=44.95,
         marker_size_mm=22.475,
         aruco_dict_tag=None,
+        using_marker_corner=False,
     ):
+        self.init_kwargs = dict(
+            square_xy=square_xy,
+            square_size_mm=square_size_mm,
+            marker_size_mm=marker_size_mm,
+            aruco_dict_tag=aruco_dict_tag,
+            using_marker_corner=using_marker_corner,
+        )
         self.square_xy = square_xy
         if aruco_dict_tag is None:
             aruco_dict_tag = cv2.aruco.DICT_4X4_250
@@ -249,24 +271,48 @@ class CharucoFeatureLib(MetaFeatureLib):
             ret, charuco_corners, charuco_ids = cv2.aruco.interpolateCornersCharuco(
                 marker_corners, marker_ids, img, self.board
             )
-            if charuco_ids is not None and len(charuco_ids) >= 11:  # 4
-                d["ids"] = charuco_ids[:, 0]
-                d["image_points"] = dict(zip(d["ids"], charuco_corners))
-                d["object_points"] = {
-                    id: self.board.chessboardCorners[id][None] for id in d["ids"]
+            if charuco_ids is not None:
+                ids = charuco_ids[:, 0]
+                image_points = dict(zip(ids, charuco_corners))
+                object_points = {
+                    id: self.board.chessboardCorners[id][None] for id in ids
                 }
-                d["corners"] = charuco_corners
+                # image_points, object_points = {}, {}
+                if self.init_kwargs.get("using_marker_corner"):
+                    marker_ids = marker_ids.squeeze()
+                    image_points.update(
+                        {
+                            -1 - i: marker_corner[0]
+                            for i, marker_corner in zip(marker_ids, marker_corners)
+                            if i < len(self.board.objPoints)
+                        }
+                    )
+                    object_points.update(
+                        {
+                            -1 - i: self.board.objPoints[i]
+                            for i in marker_ids
+                            if i < len(self.board.objPoints)
+                        }
+                    )
+                if len(image_points) >= 11:  # 4
+                    d.update(
+                        ids=ids,
+                        image_points=image_points,
+                        object_points=object_points,
+                        corners=charuco_corners,
+                    )
 
     def vis(self, d, cam=None):
         vis = super().vis(d, cam)
+        if d.get("marker_ids") is not None:
+            cv2.aruco.drawDetectedMarkers(vis, d["marker_corners"], d["marker_ids"])
         if d.get("ids") is not None:
             cv2.aruco.drawDetectedCornersCharuco(vis, d["corners"], d["ids"])
-            cv2.aruco.drawDetectedMarkers(vis, d["marker_corners"], d["marker_ids"])
         return vis
 
     @classmethod
     def build_with_calibration_img(
-        cls, hw=A4.hw, n=10, ppi=A4.ppi, aruco_dict_tag=None
+        cls, hw=A4.hw, n=10, ppi=A4.ppi, aruco_dict_tag=None, **init_kwargs
     ):
         height, width = hw
         size = max(hw) // (n)
@@ -276,7 +322,7 @@ class CharucoFeatureLib(MetaFeatureLib):
         wn = int((width) / size)
 
         square_xy = (wn, hn)
-        init_kwargs = dict(
+        init_kwargs.update(
             square_xy=square_xy,
             square_size_mm=size_mm,
             marker_size_mm=size_mm * 0.75,
@@ -284,7 +330,6 @@ class CharucoFeatureLib(MetaFeatureLib):
         )
 
         self = cls(**init_kwargs)
-        self.init_kwargs = init_kwargs
         self.calibration_img = cv2.cvtColor(
             self.board.draw(hw[::-1]), cv2.COLOR_GRAY2RGB
         )
@@ -308,6 +353,7 @@ if __name__ == "__main__":
     feature_lib = CharucoFeatureLib.build_with_calibration_img(
         aruco_dict_tag="DICT_4X4_250_start50"
     )
+    boxx.tree(feature_lib.init_kwargs)
     calibration_img = img = feature_lib.calibration_img
     # clip to 100 for printer to use less black ink
     boxx.imsave(f"/tmp/{feature_lib.calibration_img_name}", calibration_img.clip(100,))
